import 'dart:io';

import 'package:flutter/material.dart';
import 'package:android_intent/android_intent.dart';
import 'package:flutter/services.dart';

import 'package:mile_navigation_engine/mile_navigation_lib.dart';
import 'package:mile_navigation_engine_example/about.dart';
import 'package:mile_navigation_engine_example/navigation.dart';

void main() {
  runApp(
      MaterialApp(
        initialRoute: MyApp.routeName,
        routes: {
          MyApp.routeName : (context) => MyApp(),
          About.routeName : (context) => About(),
        },
      )
  );
}

class MyApp extends StatefulWidget {
  static String routeName = "/";
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {

  MileNavigationEngine _navigationEngine;

  //For iOS ONLY
  static const platform = const MethodChannel('com.mile.mile_navigation_engine_example/view_controller');

  @override
  void initState() {
    super.initState();
  }

  void goAndroidFlutterFromNavigation() async {
    AndroidIntent intent = AndroidIntent(
      action: 'android.intent.action.RUN',
      package: 'com.mile.mile_navigation_engine_example',
      componentName: 'com.mile.mile_navigation_engine_example.FlutterViewActivity',
      arguments: {'route': About.routeName},
    );
    await intent.launch();
  }

  Future<void> goIOSFlutterFromNavigation() async {
    try {
      await platform.invokeMethod('display_view_controller');
    } on PlatformException catch (e) {
      print(e.toString());
    }
  }

  @override
  Widget build(BuildContext context) {

    _navigationEngine = MileNavigationEngine(
        onNavigationFinished: (arrived) {
          print("onNavigationFinished");
        },
        onActivePOI: (result) {
          if(Platform.isAndroid) {
            print("Android POI is playing");
            //goAndroidFlutterFromNavigation();
          } else {
            print("iOS POI is playing");
            goIOSFlutterFromNavigation();
          }
        },
    );

    return Scaffold(
        appBar: AppBar(
          title: const Text('Mile Navigation Plugin'),
        ),
        body: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            InkWell(
                onTap: () {

                  String route =  '{"id":"PA20","idDestination":"V2","arrayName":{"EN":" ","FR":"Les pollinisateurs"},"arrayDescription":{"EN":" ","FR":"Laissez-vous guider à travers le parc de l\'Arquebuse et devenez incollable sur les insectes pollinisateurs !"},"arrayCoordinates":[{"latitude":47.32230128357415,"longitude":5.026825473523246},{"latitude":47.32205544620345,"longitude":5.026963160661012},{"latitude":47.32208741592632,"longitude":5.027218619180758},{"latitude":47.322126045982145,"longitude":5.027197003460458},{"latitude":47.322158015662296,"longitude":5.027022112627094},{"latitude":47.32221396255582,"longitude":5.027039798216407},{"latitude":47.32216201187086,"longitude":5.0273935100141784},{"latitude":47.32210073663859,"longitude":5.027377789490259},{"latitude":47.322086060847255,"longitude":5.027596761858973},{"latitude":47.32189823844425,"longitude":5.027555495483085},{"latitude":47.32186124460023,"longitude":5.0279676102852875},{"latitude":47.32147360874637,"longitude":5.0278831124672365},{"latitude":47.321381646238706,"longitude":5.027985367318536},{"latitude":47.32134434772868,"longitude":5.028286022346691},{"latitude":47.32089276723855,"longitude":5.028173978545823},{"latitude":47.32093939077075,"longitude":5.027761314782055},{"latitude":47.32130955319508,"longitude":5.027838291231603},{"latitude":47.3213988032162,"longitude":5.02774789821629},{"latitude":47.321434820914675,"longitude":5.027447016074603},{"latitude":47.32186374127724,"longitude":5.027545269349758},{"latitude":47.321712308326084,"longitude":5.02717561092993},{"latitude":47.32146678407389,"longitude":5.027409763690031},{"latitude":47.321077813232534,"longitude":5.027362602118444},{"latitude":47.32110845145053,"longitude":5.027170025694971},{"latitude":47.321592042654544,"longitude":5.026996786104121},{"latitude":47.321740116203785,"longitude":5.0269761952483805},{"latitude":47.321581567039374,"longitude":5.0266931563687365},{"latitude":47.32156691411228,"longitude":5.026364990423787},{"latitude":47.3218426548832,"longitude":5.026437697853794},{"latitude":47.32207576793925,"longitude":5.026632239344792}],"pois":["PO369","PO370","PO371","PO372","PO373","PO374","PO375","PO376","PO377","PO378","PO379","PO380","PO381","PO382","PO383","PO384","PO385","PO386","PO387","PO389","PO390","PO391","PO393"],"ascent":0,"length":0,"status":4,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImageRoute%2FAM%20Lasio_2_f_Lac_Kir.jpg?alt=media&token=a6fcc0c4-2a42-4727-bdfe-4b51939d716a","methodLocomotion":1,"globalAQI":0.0,"allergensAverage":0.0,"arrayShareURL":{},"isNavigationActivated":true,"areSmartFactsEnabled":true,"isStartingPointStatic":true,"isSynthActivated":true,"isLoop":false,"arrayPois":[{"id":"PO369","arrayName":{"EN":" ","FR":"Vous avez dit insectes pollinisateurs ? "},"arrayDescription":{"EN":" ","FR":"En butinant de fleurs en fleurs à la recherche de nourriture, ils transportent le pollen, assurant ainsi la reproduction d\'un grand nombre de plantes. Plus de 300 espèces ont été recensées dans les villes de France, dont au moins 191 à Dijon. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.322127,"longitude":5.026857000000007}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fautre%2FAm%20Andrena_f_Etang_Royal.jpg?alt=media&token=ffb73e73-5a5c-4279-8148-2f483434f77f","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO370","arrayName":{"EN":" ","FR":"Qui sont les pollinisateurs ? "},"arrayDescription":{"EN":" ","FR":"Les insectes pollinisateurs sont surtout des hyménoptères (abeilles, bourdons...), des diptères (mouches comme les syrphes, bombyles...), des lépidoptères (papillons) et des coléoptères (cétoines...)."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.322206,"longitude":5.027145999999988}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2Fvanessa%20atalanta1.jpg?alt=media&token=f3c2e7d7-e2b6-4de8-8279-51e4d1e4a7f5","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO371","arrayName":{"EN":" ","FR":"De nombreuses fleurs offrent le couvert"},"arrayDescription":{"EN":" ","FR":"Nectar et pollen sont les ressources alimentaires de nombreux insectes et de leurs larves.  - Le nectar est un liquide riche en sucres, acides aminés, protéines et vitamines. Toutes les plantes n’émettent pas de nectar. - Le pollen est riche en protéines, sucres, vitamines...  Les butineuses se déplacent fréquentant des fleurs « mellifères », riches en nectar, comme par exemple les pissenlits, trèfles, coquelicots, lamiers, serpolets mais aussi des fruitiers, des cultures potagères, etc."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.322065,"longitude":5.027850000000001}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fautre%2FBombus_terrestris_queen_-_Tilia_cordata_-_Keila.jpg?alt=media&token=7fc246cc-88e3-4b9d-a24c-b98fd625f766","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO372","arrayName":{"EN":" ","FR":"Des habitats très variés"},"arrayDescription":{"EN":" ","FR":"Beaucoup d\'entre nous ignorent que c\'est dans la terre que nichent la plus grande part des pollinisateurs sauvages. D\'autres construisent des nids, se servent d\'une branche, d\'un muret... La diversité des insectes pollinisateurs est liée à la diversité des habitats. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321952,"longitude":5.02788799999999}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FIMG_5332.JPG?alt=media&token=272bd259-715c-4166-91cd-c539845657aa","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO373","arrayName":{"EN":" ","FR":"Dijon métropole c\'est ma nature, j\'agis pour la biodiversité"},"arrayDescription":{"EN":" ","FR":"Pour préserver la biodiversité, il faut la connaître et pouvoir en suivre l’évolution. Cela représente une tâche immense, mais vous pouvez participer : tous ceux qui fréquentent des jardins sont invités à regarder les insectes, à les identifier et à transmettre leurs observations aux scientifiques à travers le SPIPOLL. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321927,"longitude":5.027993000000009}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2Flogo_spipoll.a25d96dd.jpg?alt=media&token=179ddae8-a4c1-4de4-9f6f-3f79536e05a5","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO374","arrayName":{"EN":" ","FR":"La pollinisation : comment ça marche ? "},"arrayDescription":{"EN":" ","FR":"1. L\'insecte vient se nourrir du pollen ou du nectar de la fleur. Des grains de pollen se fixent sur son corps.  2. L\'insecte se déplace vers une autre fleur.  3. Des grains de pollen sont déposés sur le pistil (stigmate du carpelle).  4. Si un grain de pollen est compatible, il germe et féconde l\'ovaire. Cela donnera une graine contenue dans un fruit.   En favorisant la reproduction sexuée, la pollinisation permet le brassage génétique des plantes. Or, une population de plantes diversifiée a plus de chances de résister à des agressions et de s\'adapter aux conditions changeantes du milieu dans lequel elles vivent."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321916,"longitude":5.028199000000001}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2F0001.jpg?alt=media&token=813acbdd-8159-4daa-8574-5a961e17a907","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO375","arrayName":{"EN":" ","FR":"Y\'en aura pour tout le monde !"},"arrayDescription":{"EN":" ","FR":"De nombreuses fleurs offrent un accès facile aux ressources (pollen et nectar) à un grand nombre d\'insectes différents, dits « généralistes » : les astéracées (marguerite, tournesol, bleuet...) offrent des ressources abondantes par le regroupement de nombreuses fleurs en capitules. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321772,"longitude":5.028168999999991}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fautre%2FMarguerite%20JDS.JPG?alt=media&token=be091971-8cc9-4794-a917-90d85f44ee40","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO376","arrayName":{"EN":" ","FR":"Spécialiste de la figue"},"arrayDescription":{"EN":" ","FR":"Le figuier est un cas particulier : la fécondation et le développement des figues dépend uniquement d\'une petite « guêpe » de sexe femelle, Blastophaga psenes L., qui seule peut apporter le pollen dans la figue. Elle se reproduit à l\'intérieur de la figue et y passe la majeure partie de sa vie. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321499,"longitude":5.027981000000011}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FBlastophaga%20psenes1%20-%20fourni%20par%20Michel%20DESCAMPS.jpeg?alt=media&token=f99b6f50-5e62-43c6-8610-7a033a7479e7","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO377","arrayName":{"EN":" ","FR":"Humm... de la viande pourrie !"},"arrayDescription":{"EN":" ","FR":"Certaines plantes émettent des signaux olfactifs attirant certains insectes. Ainsi l\'Arum italicum attire des insectes coprophages, notamment des « mouches » (diptères).  Certaines espèces de diptères « à antennes courtes » se nourrissent de nectar et de pollen, avec une trompe adaptée à leur régime alimentaire. Certains diptères (mouche bleue, grand bombyle) jouent probablement un rôle important pour la pollinisation des petites fleurs peu attractives pour les gros pollinisateurs. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321446,"longitude":5.028168999999991}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FSP%20Calliphora%20vicina%20Robineau-Desvoidy.JPG?alt=media&token=eb7dac2d-1e02-4a34-b685-575a2daf38ad","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO378","arrayName":{"EN":" ","FR":"Aides de champs "},"arrayDescription":{"EN":" ","FR":"Le Syrphe ceinturé a une action de pollinisation sur des plantes cultivées, ou proches des cultures. De plus, les larves se nourrissent de pucerons (lutte biologique) : l\'adulte pond ses œufs à proximité des colonies de pucerons, y compris sur les cultures."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321126,"longitude":5.028064999999998}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FHoverfly_November_2007-3.jpg?alt=media&token=2c7c9939-8fd1-4cc6-b153-1719b986eb3e","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO379","arrayName":{"EN":" ","FR":"Les coléoptères, floricoles depuis des millions d\'année"},"arrayDescription":{"EN":" ","FR":"Parmi les premiers insectes floricoles connus, il y a 200 millions d’années (Jurassique inférieur ou Hettangien), des petits coléoptères fréquentaient des cycas. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321008,"longitude":5.027974999999998}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FIMG_2865.JPG?alt=media&token=d93fb9eb-c323-4b96-a492-c3c388d426d9","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO380","arrayName":{"EN":" ","FR":"Piste d\'atterrissage droit devant !"},"arrayDescription":{"EN":" ","FR":"Les insectes sont attirés par les signaux émis par les fleurs, notamment visuels. Les insectes, sensibles aux ultra-violets ne perçoivent pas les couleurs de la même manière que les humains. Les pétales de certaines fleurs sont ornementés de lignes, de taches, de points servant à guider l\'insecte vers le nectar (guides nectarifères). "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.320957,"longitude":5.028020999999995}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FAquile%20filipendule%20JDS.JPG?alt=media&token=db9c51d5-0849-430b-a52a-c1438c604876","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO381","arrayName":{"EN":" ","FR":"Comment favoriser les pollinisateurs en ville ?"},"arrayDescription":{"EN":" ","FR":"La présence de nombreuses espèces de pollinisateurs en abondance est un indicateur de la qualité de l\'environnement, y compris dans les villes. Un portail citoyen permet à tous les dijonnais d\'agir et de favoriser la biodiversité.  Depuis plusieurs années à Dijon, un plan pour la biodiversité mobilise gestionnaires, aménageurs et habitants pour favoriser la biodiversité dans la ville.    Pour tout savoir et agir rendez-vous sur : www.ma-nature.dijon.fr "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321057,"longitude":5.028379000000001}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fautre%2FSP%20Anthophora%20plumipes%20mele.JPG?alt=media&token=17e6f0bb-1a19-482a-af67-ed17db79611f","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO382","arrayName":{"EN":" ","FR":"La diversité des pollens influence le goût du miel "},"arrayDescription":{"EN":" ","FR":"Parce qu\'elles n\'y trouvent par les mêmes fleurs, les abeilles fréquentant les différents quartiers de Dijon produisent des miels aux goûts différents. Mieux connaître leurs habitudes alimentaires permet de cibler les plantations mellifères et les aménagements à prévoir. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321126,"longitude":5.0275629999999865}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fautre%2Fplan-ruchers-2018.jpeg?alt=media&token=ec245b0a-5224-4146-9766-d6f4e7c8c692","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO383","arrayName":{"EN":" ","FR":"Bourdons : super-champions !"},"arrayDescription":{"EN":" ","FR":"Les bourdons étant très velus, le pollen se fixe davantage à leurs soies. Ils butinent plus longtemps dans la journée et dans des conditions climatiques plus défavorables que les abeilles domestiques. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321391,"longitude":5.0276560000000075}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2Fbourdon%20sur%20cardon%20-%20JDS.JPG?alt=media&token=7719a88c-2f74-44b7-95db-2f57c882d705","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO384","arrayName":{"EN":" ","FR":"Pollen déposé à coup sûr !"},"arrayDescription":{"EN":" ","FR":"Au fil de l\'Evolution, différentes stratégies sont apparues chez les plantes pour favoriser la transmission du pollen, c\'est à dire son dépôt sur l\'insecte sur une fleur, puis par l\'insecte sur une autre fleur.   Certaines émettent de grandes quantités de pollen (Lychnis), d\'autres disposent de systèmes mécaniques répondant au passage d\'un insecte :  étamines mobiles (Heliantheme des apennins, Ruta graveolens), système de balancier (Salvia pratensis) ou encore explosion des étamines (Spartim junceum)."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321482,"longitude":5.027639999999991}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fautre%2FRuta%20angustifolia.jpg?alt=media&token=441614ac-670e-42af-b161-ef18723a77c2","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO385","arrayName":{"EN":" ","FR":"+ de pollinisateurs =  + de rendements"},"arrayDescription":{"EN":" ","FR":"Lorsque le nombre et la diversité des espèces d’insectes pollinisateurs sont plus importants, le rendement des cultures est accru de plus de 20 % en moyenne."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321743,"longitude":5.0277969999999925}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2Fchamp%20moutarde%20melanie%20durafour.jpg?alt=media&token=a775230f-a374-4065-8de9-a984dd92755a","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO386","arrayName":{"EN":" ","FR":"Le déclin des pollinisateurs affectent les cultures"},"arrayDescription":{"EN":" ","FR":"Le phénomène ne touche pas seulement les abeilles domestiques. Actuellement, toutes les études pointent le déclin des populations et de la diversité des espèces d\'insectes pollinisateurs, en particulier des « spécialistes ».  Les changements d\'occupation du sol, en particulier l\'urbanisation, restent la cause majeure du déclin, en raison d\'un déficit en sites d\'alimentation et de nidification. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.32172,"longitude":5.0277969999999925}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fautre%2FDSC08243.JPG?alt=media&token=e5eeec70-f271-4881-9334-e580e863bdad","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO387","arrayName":{"EN":" ","FR":"Une question de spécialistes"},"arrayDescription":{"EN":" ","FR":"Certaines fleurs, comme celles des fabacées, offrent un accès difficile aux ressources (pollen et nectar) : ils présentent des caractères (tube nectarifère allongé, période de floraison précoce, courte...) attirant des insectes « spécialistes ». La plupart des espèces de papillons (lépidoptères) fréquentent les fleurs : elles y récoltent le nectar dont elles se nourrissent. Les papillons de nuit sont des pollinisateurs plus nombreux que les papillons de jour. La piéride fréquente les plantes de la famille des choux car ses larves s\'en nourrissent."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321806,"longitude":5.0276700000000005}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FVanessa_cardui_on_Lavandula_angustifolia-2459.jpg?alt=media&token=5037727c-6736-4003-9f58-7b4d52f0bbfc","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO389","arrayName":{"EN":" ","FR":"Mon jardin est accueillant "},"arrayDescription":{"EN":" ","FR":"Tous les espaces verts ne sont pas favorables aux pollinisateurs. Dans ce domaine, chacun peut agir, en ménageant des espaces pour nidifier, des ressources alimentaires adaptées, des corridors pour se déplacer et surtout en abandonnant les produits nocifs pour l\'environnement.  Sur le territoire de Dijon métropole, les boisements, les pelouses sèches et les prairies sont des réservoirs de biodiversité pour les abeilles sauvages. Les berges de rivières constituent à la fois des lieux de vie et des corridors pour la circulation des insectes. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321658,"longitude":5.0270730000000015}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FDSC_0045.JPG?alt=media&token=b632891f-358d-45fb-b444-039ad6d50a22","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO390","arrayName":{"EN":" ","FR":"L\'osmie, une des 200 espèces d\'abeilles sauvages à Dijon"},"arrayDescription":{"EN":" ","FR":"Les osmies sont des abeilles sauvages solitaires. Parfois appelées « maçonnes », elles nichent dans des cavités variées. Actives une grande partie de l\'année, elles jouent un rôle important dans la pollinisation. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321624,"longitude":5.027031999999991}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2Fosmia%20bicornis1.jpg?alt=media&token=c5356ca6-3d02-4a12-adf3-996784e15e68","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO391","arrayName":{"EN":" ","FR":"L\'Abeille mellifère, le plus connu des pollinisateurs"},"arrayDescription":{"EN":" ","FR":"Sélectionnées et élevées par l\'homme, elles vivent en colonies dans des ruches. Les butineuses visitent les fleurs dans un rayon de 1 à 3 km autour de la ruche, à la recherche de nectar (servant à produire le miel) et de pollen qu\'elles récoltent dans leurs « corbeilles », essentiellement pour nourrir leurs larves."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321744,"longitude":5.026605999999987}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2Fabeille%20domestique1.jpg?alt=media&token=119da068-3014-4527-8f39-6c2979844da5","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4},{"id":"PO393","arrayName":{"EN":" ","FR":"Oh, une Cétoine dorée !"},"arrayDescription":{"EN":" ","FR":"Certains coléoptères, comme la Cétoine dorée, sont de gros consommateurs de nectar, mais ils sont généralement des pollinisateurs peu efficaces par rapport aux autres groupes d’insectes floricoles (qui fréquentent les fleurs). "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.322077,"longitude":5.027335999999991}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FSJ%20Cetonia%20aurata%2010.JPG?alt=media&token=6a53f7c2-9372-4c74-9f82-4b14c93850fd","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4}]}';

                  _navigationEngine.startNavigation(
                    route: route,
                    gpsColor: "#ff000000",
                    accessToken: "pk.eyJ1IjoibWlsZS1kZXYiLCJhIjoiY2s0OGJkNWEyMDB5ZTNna2Z5b2c2YTF3MCJ9.D5Ebdy0_ADSTQPTH4GCc9w",
                    mode: NavigationMode.NAVIGATE_IN_ROUTE,
                  );
                },
                child: Container(
                  height: 36,
                  width: 200,
                  color: Colors.blue,
                  child: Center(
                    child: Text(
                      "Navigation In Route",
                      style: TextStyle(
                        color: Colors.white,
                      ),
                    ),
                  ),
                )
            ),
            Container(height: 20,),
            InkWell(
                onTap: () {
                  String route = '{"id":"PA20","idDestination":"V2","arrayName":{"EN":" ","FR":"Les pollinisateurs"},"arrayDescription":{"EN":" ","FR":"Laissez-vous guider à travers le parc de l\'Arquebuse et devenez incollable sur les insectes pollinisateurs !"},"arrayCoordinates":[{"latitude":47.32230128357415,"longitude":5.026825473523246},{"latitude":47.32205544620345,"longitude":5.026963160661012},{"latitude":47.32208741592632,"longitude":5.027218619180758},{"latitude":47.322126045982145,"longitude":5.027197003460458},{"latitude":47.322158015662296,"longitude":5.027022112627094},{"latitude":47.32221396255582,"longitude":5.027039798216407},{"latitude":47.32216201187086,"longitude":5.0273935100141784},{"latitude":47.32210073663859,"longitude":5.027377789490259},{"latitude":47.322086060847255,"longitude":5.027596761858973},{"latitude":47.32189823844425,"longitude":5.027555495483085},{"latitude":47.32186124460023,"longitude":5.0279676102852875},{"latitude":47.32147360874637,"longitude":5.0278831124672365},{"latitude":47.321381646238706,"longitude":5.027985367318536},{"latitude":47.32134434772868,"longitude":5.028286022346691},{"latitude":47.32089276723855,"longitude":5.028173978545823},{"latitude":47.32093939077075,"longitude":5.027761314782055},{"latitude":47.32130955319508,"longitude":5.027838291231603},{"latitude":47.3213988032162,"longitude":5.02774789821629},{"latitude":47.321434820914675,"longitude":5.027447016074603},{"latitude":47.32186374127724,"longitude":5.027545269349758},{"latitude":47.321712308326084,"longitude":5.02717561092993},{"latitude":47.32146678407389,"longitude":5.027409763690031},{"latitude":47.321077813232534,"longitude":5.027362602118444},{"latitude":47.32110845145053,"longitude":5.027170025694971},{"latitude":47.321592042654544,"longitude":5.026996786104121},{"latitude":47.321740116203785,"longitude":5.0269761952483805},{"latitude":47.321581567039374,"longitude":5.0266931563687365},{"latitude":47.32156691411228,"longitude":5.026364990423787},{"latitude":47.3218426548832,"longitude":5.026437697853794},{"latitude":47.32207576793925,"longitude":5.026632239344792}],"pois":["PO376"],"ascent":0,"length":0,"status":4,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImageRoute%2FAM%20Lasio_2_f_Lac_Kir.jpg?alt=media&token=a6fcc0c4-2a42-4727-bdfe-4b51939d716a","methodLocomotion":1,"globalAQI":0.0,"allergensAverage":0.0,"arrayShareURL":{},"isNavigationActivated":true,"areSmartFactsEnabled":true,"isStartingPointStatic":true,"isSynthActivated":true,"isLoop":false,"arrayPois":[{"id":"PO376","arrayName":{"EN":" ","FR":"Spécialiste de la figue"},"arrayDescription":{"EN":" ","FR":"Le figuier est un cas particulier : la fécondation et le développement des figues dépend uniquement d\'une petite « guêpe » de sexe femelle, Blastophaga psenes L., qui seule peut apporter le pollen dans la figue. Elle se reproduit à l\'intérieur de la figue et y passe la majeure partie de sa vie. "},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":47.321499,"longitude":5.027981000000011}],"isMP3Activated":false,"imageURL":"https://firebasestorage.googleapis.com/v0/b/dijon-metropole.appspot.com/o/ImagePOI%2Fimg%2FBlastophaga%20psenes1%20-%20fourni%20par%20Michel%20DESCAMPS.jpeg?alt=media&token=f99b6f50-5e62-43c6-8610-7a033a7479e7","radius":10,"type":7,"isAutoDirectionAcivated":true,"status":4}]}';

                  _navigationEngine.startNavigation(
                    route: route,
                    gpsColor: "#ff000000",
                    accessToken: "pk.eyJ1IjoibWlsZS1kZXYiLCJhIjoiY2s0OGJkNWEyMDB5ZTNna2Z5b2c2YTF3MCJ9.D5Ebdy0_ADSTQPTH4GCc9w",
                    mode: NavigationMode.NAVIGATE_TO_POI,
                  );
                },
                child: Container(
                  height: 36,
                  width: 200,
                  color: Colors.blue,
                  child: Center(
                    child: Text(
                      "Navigation To Poi",
                      style: TextStyle(
                        color: Colors.white,
                      ),
                    ),
                  ),
                )
            ),
            Container(height: 20,),
            InkWell(
                onTap: () {
                  String route = '{"id":"PAG1","idDestination":"V0","routeRadius":150,"routeCenter":{"latitude":43.563829368553584,"longitude":5.022701746051467},"arrayName":{"EN":"Village treasure hunt","FR":"Jeu de piste dans le village"},"arrayDescription":{"EN":"Head out and explore Miramas-le-Vieux on this medieval adventure. Scan the QR codes located near points of interest to gain access to mystery questions.","FR":"Partez à la découverte de Miramas-le-Vieux à travers cette aventure médiévale. Scannez les QR codes disposés à proximité des points d’intérêt pour accéder aux énigmes."},"pois":["PO120","PO121","PO122","PO123","PO124","PO125","PO126","PO127","PO128","PO129","PO130","PO131","PO132"],"ascent":30,"length":1360,"status":4,"imageURL":"https://firebasestorage.googleapis.com/v0/b/miramas-498e5.appspot.com/o/ImageRoute%2Fau-coeur-du-village-medieval.jpg?alt=media&token=468d815e-78bb-4474-ab81-cd6d6822bcf6","averageDuration":45,"methodLocomotion":1,"globalAQI":0.0,"allergensAverage":0.0,"allergensInRoute":{},"arrayShareURL":{},"isNavigationActivated":true,"areSmartFactsEnabled":true,"isStartingPointStatic":true,"isSynthActivated":false,"isLoop":true,"arrayPois":[{"id":"PO120","arrayName":{"EN":"Step 1 - Jubilee Cross","FR":"Etape 1 - Croix du Jubilé"},"arrayDescription":{"EN":"Did you know? The Jubilee Cross is located on the remains of a cemetery. However, the tombs, carved out of rock following the customs of the time, slowly disappeared over time because of high foot traffic. Today the land has been transformed into a parking lot.","FR":"Le saviez-vous ? La Croix du Jubilé se trouve sur les vestiges d’un cimetière. À l’intérieur des tombes, taillées dans le roc suivant l’usage de l’époque et visitées au cours des siècles, il ne reste plus rien. Un parking recouvre dorénavant l’ensemble."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.563998,"longitude":5.022300999999999}],"isMP3Activated":false,"radius":20,"type":1,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Head towards the village. You will see a staircase on your left. This is where you next clue is located.","FR":"Dirigez-vous vers le village. Un escalier se trouvera sur la gauche. Cherchez-y le prochain indice."},"arrayQuestions":{"EN":"Let\'s get this adventure started. This is the Jubilee Cross. What year was it built in?","FR":"Voici le départ de notre aventure. Ceci est la Croix du Jubilé. En quelle année a-t-elle été érigée ?"},"arrayAnswers":{"EN":"1874","FR":"1874"},"arrayHints":{"EN":"Take a good look at the steps around the monument.","FR":"Regardez bien les marches tout autour du monument."},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/croix"}},{"id":"PO121","arrayName":{"EN":"Step 2 - The Staircase ","FR":"Etape 2 - L\'escalier"},"arrayDescription":{"EN":"The Staircase of Sighs is the most direct way of reaching the summit of this perched village. These steps also lead to the Cercle de l\'Avenir, a meeting point that comes to life during the Saint-Julien festival, commemorating the village\'s patron saint. During this event in late August, the villagers gather and share a traditional meal of aïoli, a type of garlic mayonnaise served with vegetables and fish.","FR":"Le saviez-vous ? L’escalier des soupirs permet une ascension plus directe vers le sommet du village perché. C’est également par cette ruelle que l’on accède au Cercle de l’Avenir, lieu de réunion qui atteint son apogée pour la fête de Saint-Julien, patron du village. Lors de cet événement qui a lieu fin août, les habitants ont pour coutume de partager l’aïoli traditionnel, préparation ressemblant à une mayonnaise à l’ail, servie avec des légumes et du poisson."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.563274,"longitude":5.023470000000003}],"isMP3Activated":false,"radius":15,"type":0,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Don\'t take the Staircase of Sighs. Continue on the main road towards the village school. The next clue is hiding there.","FR":"N’empruntez pas l’escalier des soupirs. Continuez votre parcours sur la route principale jusqu’à l’école du village, le prochain indice s’y cache."},"arrayQuestions":{"EN":"This staircase wears its name well. In fact, it references one of the famous bridges of Venice. Which one exactly? Staircase of... ","FR":"Cet escalier porte bien son nom, qui peut également faire référence à un des ponts célèbres de Venise. Quel est-il ? Escalier des..."},"arrayAnswers":{"EN":"sighs","FR":"soupirs"},"arrayHints":{"EN":"Look at the plate above the QR code.","FR":"Regardez la plaque au-dessus du QR code."},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/escalier"}},{"id":"PO122","arrayName":{"EN":"Step 3 - Village school ","FR":"Etape 3 - L\'école du village"},"arrayDescription":{"EN":"Did you know? The village school opened in 1866 in Miramas-le-Vieux. The first public school in the village for only girls. The boys, on the other hand, studied in the adjacent building. Over time, people moved away from the village and towards the big city. In order to maintain the education system, an essential part of village life, the municipality decided to combine the two establishments to form the first coed school with mixed grades.  ","FR":"Le saviez-vous ? L’école du village vit le jour en 1866. Première école communale gratuite, elle accueillait uniquement les filles. Les garçons, quant à eux, étudiaient dans le bâtiment voisin. Au fil des années, la commune subit un exode de sa population qui migra vers la ville nouvelle. Afin de maintenir l’éducation, source de vie pour le village, la municipalité décida de fusionner les deux établissements pour former la première école mixte à classe unique."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.562494,"longitude":5.024369000000007}],"isMP3Activated":false,"radius":15,"type":1,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Head straightforward about 10 metres to find the next clue. ","FR":"Avancez d’une dizaine de mètres pour découvrir le prochain indice."},"arrayQuestions":{"EN":"Here\'s the old girls school, it\'s now the village school. In your opinion, how many classes does this school have today?","FR":"Voici l’ancienne école de filles, c’est maintenant l’école du village. À votre avis, combien de classes occupent cette école aujourd’hui ?"},"arrayAnswers":{"EN":"1","FR":"1"},"arrayHints":{"EN":"Less than 2","FR":"Moins de 2"},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/ecole"}},{"id":"PO123","arrayName":{"EN":"Step 4 - The oil mill","FR":"Etape 4 - Le moulin à huile"},"arrayDescription":{"EN":"The olive tree was considered « the first of all trees ». With olives, one could make oil, soap and fertiliser. Unfortunately, these olive orchards did not stand the test of time. In fact, several natural disasters slowly ate away at the orchards, which were never replanted. The cold spell of 1956 proved fatal, killing off the last orchards in Miramas.","FR":"L\'olivier était un arbre très important. Il était considéré comme « le premier de tous les arbres ». On en faisait de l’huile, du savon et de l’engrais. Les vergers d’oliviers n’ont hélas pas survécu au temps à cause de plusieurs désastres naturels, et ils ne furent pas replantés. La vague de froid de 1956 fut fatale aux derniers oliviers de Miramas."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.562497,"longitude":5.024788000000001}],"isMP3Activated":false,"radius":25,"type":0,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Now head to Castagne square and enjoy the fountain. The next clue is hiding on the side with the missing head.","FR":"Rendez-vous maintenant sur la place Castagne et admirez la fontaine. Du côté de la tête manquante se cache le prochain indice."},"arrayQuestions":{"EN":"Here you are, in front of the mill. Black or green, my oil can do wonders! Who am I?","FR":"Vous voici à coté du moulin. Noire ou verte, mon huile fait des merveilles ! Qui suis-je ?"},"arrayAnswers":{"EN":"Olives","FR":"Olive"},"arrayHints":{"EN":"I can be consumed as a spread on toast, too!","FR":"On me consomme également en tapenade !"},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/le-moulin"}},{"id":"PO124","arrayName":{"EN":"Step 5 - Castagne square","FR":"Etape 5 - La place Castagne"},"arrayDescription":{"EN":"Did you know? Castagne square gets its name from Jean Louis Martin Castagne, the former village mayor, who managed to build a train station on the Marseille-Avignon railway in 1845. He also spearheaded the fountain project to bring water to the city centre. His generosity towards Miramas can be seen in his legacy. To honour him, his successor built a bust in 1860 over the village fountain that he had yearned for. Behind the bust, leaning against the rock, you\'ll find the wash house. ","FR":"Le saviez-vous ? La place Castagne tient son nom de Jean Louis Martin Castagne, ancien maire du village. C\'est lui qui obtint la construction d’une gare sur la ligne Marseille-Avignon en 1845. Il fut également l\'initiateur du projet de fontaine pour amener l’eau au cœur du village. Sa générosité envers Miramas se manifesta une dernière fois au travers de son testament. En remerciement, le successeur de Castagne fit ériger un buste qui trône depuis 1860 sur la fontaine du village qu’il avait tant souhaitée. Derrière lui, adossé au rocher, on découvre le lavoir."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.562742,"longitude":5.024892999999992}],"isMP3Activated":false,"radius":15,"type":5,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Turn the wheels again for future adventurers. Then, get back on the route towards Rue Mireille. Your next clue awaits you there!","FR":"Mélangez de nouveau les roues pour les futurs aventuriers. Puis reprenez votre chemin vers la calade baptisée rue Mireille. Le prochain indice se trouve à l’entrée de celle-ci."},"arrayQuestions":{"EN":"The old telephone booth is home to this mystery. Bring the water from the spring to the fountain. Be careful, the water has to travel through the pipes and can\'t be lost. Once the pipes are connected, each colour marker will give you a letter.","FR":"L’ancienne cabine téléphonique abrite l’énigme. Amenez l’eau de la source jusqu’à la fontaine. Attention, l’eau doit circuler par tous les tuyaux et ne doit pas être perdue. Une fois les tuyaux reliés, chaque marqueur de couleur vous donnera une lettre."},"arrayAnswers":{"EN":"Wash house","FR":"Lavoir"},"arrayHints":{"EN":"Green - Yellow - Red - Blue - Violet - White","FR":"Vert – Jaune – Rouge – Bleu - Violet – Blanc"},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/place-castagne"}},{"id":"PO125","arrayName":{"EN":"Step 6 - The silo","FR":"Etape 6 - Le silo"},"arrayDescription":{"EN":"Did you know? The silo made it possible to stock grains in a cool area for several years away from the air and light. In fact, at the time most homes were equipped with them. The location of one of these silos is still visible today on this yellow-coloured smalt cliff, which is very typical of the Provence region. ","FR":"Le saviez-vous ? Le silo permettait de stocker le grain dans un endroit frais, à l’abri de l’air et de la lumière, pendant plusieurs années. On en trouvait dans la plupart des maisons de l’époque. L’emplacement d’un de ces silos est encore visible dans cette falaise de safre, roche de couleur jaune, typique en Provence."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.562805,"longitude":5.024696000000006}],"isMP3Activated":false,"radius":15,"type":0,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Continue your climb to the Notre-Dame door. Don\'t forget to check out, on your left, the house dating back to the Renaissance.","FR":"Poursuivez votre ascension jusqu’à la porte Notre-Dame. N’oubliez pas d’admirer, sur votre gauche, une maison datant de la Renaissance."},"arrayQuestions":{"EN":"Raise your head and look at the cavity carved into the rock. Solve the puzzle to find out what it stands for. 19, 9, 12, 15","FR":"Levez la tête pour apercevoir le creux taillé dans la roche. Résolvez l’énigme pour savoir ce qu’il représente. 19, 9, 12, 15"},"arrayAnswers":{"EN":"Silo","FR":"Silo"},"arrayHints":{"EN":"1 = a","FR":"1 = a"},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/le-creux"}},{"id":"PO126","arrayName":{"EN":"Step 7 - The Aleppo pine","FR":"Etape 7 - Le pin d\'Alep"},"arrayDescription":{"EN":"Did you know? The Aleppo pine, native to the Mediterranean region, offers a warm welcome to walkers and runners alike in Miramas-le-Vieux. Its branches, resting on the stone of the Notre-Dome door, cover an entire bend. A must-see part of the local heritage, it won the title Impressive Tree of France in 2000, as a reward for the work done by the municipality in maintaining, preserving and saving it.","FR":"Le saviez-vous ? Le pin d’Alep, arbre méditerranéen plus que centenaire, offre une haie d’honneur toujours verte aux promeneurs de Miramas-le-Vieux. Ses branches, en appui sur la pierre de la porte Notre-Dame, enveloppent tout un virage. Élément incontournable du patrimoine local, il a reçu le label Arbre remarquable de France en l’an 2000, récompensant le travail accompli par la municipalité pour l’entretenir, le préserver et le sauvegarder."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.564074,"longitude":5.024013999999994}],"isMP3Activated":false,"radius":15,"type":0,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Continue on your way to make it to the walls of the castle. Find the bronze bust, it will show you the way to Salomon square. In this square, your next mystery awaits!","FR":"Continuez votre chemin pour atterrir dans l\'enceinte du château. Repérez le buste en bronze, il vous montre l\'entrée de la place Salomon. Sur cette place se trouve votre prochaine énigme."},"arrayQuestions":{"EN":"The Notre-Dame door once was the castle\'s main entrance. A twisted 100 year old pine tree has now created a nice shady area. Find out in what year this tree received the award for the most impressive tree in France to continue along on the adventure.","FR":"La porte Notre-Dame était autrefois l’entrée principale du château. Un pin centenaire et bistourné lui fait aujourd’hui une ombre légère. Trouvez en quelle année il a reçu le label Arbre remarquable de France pour continuer l’aventure."},"arrayAnswers":{"EN":"2000","FR":"2000"},"arrayHints":{"EN":"Look at the plate on the wall.","FR":"Observez la plaque apposée sur le mur."},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/porte-et-pin"}},{"id":"PO127","arrayName":{"EN":"Step 8 - The castle","FR":"Etape 8 - Le château"},"arrayDescription":{"EN":"Did you know? The castle, most likely from the 10th century, was built after the surrounding plains were abandoned by the Miramasséens in order to protect themselves. Towards 1096, Guillaume the 1st gifted castra and seigneury to the abbey of Montmajour. It was pillaged and besieged during the Wars of Religion, until it was abandoned in 1850. Over time, different parts of the castle were turned into living quarters or used for farming. Starting in the 1970s, the village was repopulated and the locals restored the castle in order to save the last remaining relics from the Romanesque period.","FR":"Le saviez-vous ? Le château, datant probablement du 10ème siècle, fut édifié suite à l’abandon de la plaine par les Miramasséens dans le but de se protéger. Vers 1096, Guillaume Ier fit don du castrum et de la seigneurie à l’abbaye de Montmajour. Il fut pillé et assiégé pendant les guerres de religion, jusqu’à son abandon en 1850. Au fil du temps, diverses parties du château furent transformées en habitations ou en fermes. À partir des années 1970, le village se repeupla et les habitants entreprirent des travaux de restauration pour sauvegarder les derniers vestiges de l’époque romane."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.5642,"longitude":5.024509999999992},{"latitude":43.564097,"longitude":5.024360999999999}],"isMP3Activated":false,"radius":10,"type":1,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Get back on the path and find Place des Trambourinaires for access to the next mystery. It\'s located behind the shop Envie de Provence under the archway.","FR":"Reprenez votre chemin et trouvez la place des Tambourinaires pour accéder à la prochaine énigme. Elle se situe derrière la boutique Envie de Provence, sous le porche."},"arrayQuestions":{"EN":"Two birds of a feather. Will you be able to figure out the secrets of the wheel as well as find out who Guillaume the 1st, seigneur of Baussenc, gifted the castra and seigneury to? Wine - Helmet - Olive branch - Crown - Sword - Knight","FR":"Les 2 font la paire. Saurez-vous percer les secrets de la roue et ainsi trouver à qui Guillaume Ier, seigneur de Baussenc, a fait don du castrum et de la seigneurie ? Vin – Casque - Rameau d\'olivier – Couronne – Épée - Chevalier"},"arrayAnswers":{"EN":"Monks","FR":"Moines"},"arrayHints":{"EN":"Make 2 identical symbols appear and discover a letter.","FR":"Faites apparaître 2 symboles identiques pour découvrir une lettre."},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/place-salomon"}},{"id":"PO128","arrayName":{"EN":"Step 9 - La Place des Tambourinaires","FR":"Etape 9 - La place des Tambourinaires"},"arrayDescription":{"EN":"Did you know? The original coat of arms of Miramas was a blue shield. The letter M was put on it for Miramas, Miramaris in the Middle Ages. The key states that the city is fortified and that it belongs to the Abbey of Montmajour. Besans, an ancient currency, remind us of the seigneur of Miramaris\' participation in crusades and represent prosperity. ","FR":"Le saviez-vous ? Le blason originel de Miramas est composé d’un écu de couleur azur. La lettre M indique Miramas, Miramaris au Moyen Âge. La clé signifie que la cité est fortifiée et qu\'elle appartient à l’abbaye de Montmajour. Les besans rappellent la participation du seigneur de Miramaris aux croisades, et sont un signe de prospérité."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.564052,"longitude":5.024764000000005}],"isMP3Activated":false,"radius":15,"type":5,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Before getting back on the main path, and continuing on with the adventure to Notre-Dame-de-Beauvezer Church, enjoy the square. You may not think it but this public square is a place of wonder and fun.","FR":"Avant de revenir sur l’axe principal du village et de poursuivre votre aventure jusqu’à l’église Notre-Dame-de-Beauvezer, admirez la place des Tambourinaires. Contrairement à ce que l’on pourrait penser, c’est un lieu public où l’émerveillement et la convivialité sont au rendez-vous."},"arrayQuestions":{"EN":"Reconstruct the coat of arms on the low wall and find out in what year the banner was added to the bottom of it.","FR":"Reconstituez le blason apposé sur le muret et trouvez en quelle année a été rajoutée l’oriflamme sous le blason."},"arrayAnswers":{"EN":"1950","FR":"1950"},"arrayHints":{"EN":"Visual of the coat of arms","FR":"Visuel du blason"},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/tambourinaires"}},{"id":"PO129","arrayName":{"EN":"Step 10 - Notre-Dame-de-Beauvezer Church","FR":"Etape 10 - L\'église Notre-Dame-de-Beauvezer"},"arrayDescription":{"EN":"Did you know? The Notre-Dame-de-Beauvezer Church is located within the village. It was built in the 15th century in honour of the Virgin Mary. Later, Saint Vincent took Mary\'s place as protector. In an alcove above the front gate sits a statue of the Madonna and Child, topped with a coat of arms of the Abbey of Montmajour. One can appreciate the coat of arms, in colour, inside the church near the centre. Even after being destroyed multiple times during the Wars of Religion, the church was rebuilt in 1574 and then completely restored in 1978.  ","FR":"Le saviez-vous ? L’église Notre-Dame-de-Beauvezer est l’église intramuros du village. Elle fut construite au 15ème siècle en l’honneur de la Vierge Marie. Plus tard, elle prit saint Vincent comme protecteur. Au-dessus du portail, une niche abrite une statue de la Vierge à l’Enfant. Celle-ci est surmontée d’un écusson aux armoiries de l’abbaye de Montmajour. On retrouve ces armoiries, en couleur, à l’intérieur de l’église au niveau de la clé de voûte. Plusieurs fois détruite lors des guerres de religion, l’église fut reconstruite en 1574 puis entièrement restaurée en 1978."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.563641,"longitude":5.024637000000013}],"isMP3Activated":false,"radius":15,"type":6,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Continue along the path until you arrive at the Chapel of Pénitents Noirs.","FR":"Continuez votre chemin jusqu’à la chapelle des Pénitents Noirs."},"arrayQuestions":{"EN":"Look at the child at the top of the door, it would appear he is missing something, but what?","FR":"Observez l’enfant au dessus du portail, il semblerait qu’il lui manque quelque chose, mais quoi ?"},"arrayAnswers":{"EN":"Head","FR":"Sa tête"},"arrayHints":{"EN":"It looks like someone lost their...","FR":"Attention à ne pas la perdre..."},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/beauvezer"}},{"id":"PO130","arrayName":{"EN":"Step 11 - La chapelle des Pénitents Noirs ","FR":"Etape 11 - La chapelle des Pénitents Noirs"},"arrayDescription":{"EN":"Did you know? The Chapel of the Black Penitents used to be home to a brotherhood, which practiced Catholicism. The members would march wearing their heavy wooden crosses, dressed in black masks. Of simple and elegant construction, the chapel was once the location of the village\'s first town hall. However, in 1894, the town hall was relocated to Miramas Gare, a historic turning point that marked the beginning of contemporary history.  ","FR":"Le saviez-vous ? La chapelle des Pénitents Noirs accueillait en son temps une confrérie pratiquant la religion catholique. Les membres qui la composaient défilaient vêtus de cagoules noires, en portant leur lourde croix de bois. De composition simple et élégante, la chapelle abrita ensuite la toute première mairie du village à l’étage. Le transfert officiel de cette dernière vers Miramas Gare, en 1894, fut un tournant historique qui marqua le début de l’histoire contemporaine."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.563228,"longitude":5.024518999999998}],"isMP3Activated":false,"radius":15,"type":6,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Head down the alley until you get to the public garden.","FR":"Descendez maintenant la ruelle jusqu’au jardin public."},"arrayQuestions":{"EN":"Take the staircase behind you and enjoy the landscape. Give the viewpoint indicator a good look. In what year was the town hall officially moved to Miramas Gare?","FR":"Prenez l’escalier derrière vous pour admirer le paysage et observez bien la table d’orientation. En quelle année fut transférée officiellement la mairie vers Miramas Gare ?"},"arrayAnswers":{"EN":"1894","FR":"1894"},"arrayHints":{"EN":"The text to the right side of the viewpoint indicator will help you.","FR":"Le texte situé sur la droite de la table d\'orientation vous aidera."},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/penitents"}},{"id":"PO131","arrayName":{"EN":"Step 12 - The public garden","FR":"Etape 12 - Le jardin public"},"arrayDescription":{"EN":"Did you know? The fig, which has been around since the Antiquity and seen as a staple of basic nutrition in the Mediterranean since ancient times, is full of benefits and nutrition, like olives and even grapes. Rich in fibre, beta carotene, polyphenols and minerals, it is invigorating, nourishing, full of antioxidants and minerals, a laxative and a protector of the vascular system. Very digestible, try this miraculous fruit in many different ways; raw, cooked, or dried.","FR":"Le saviez-vous ? Connue depuis l’Antiquité comme l’un des fruits élémentaires à l’alimentation des anciennes civilisations du bassin méditerranéen, au même titre que la datte, l’olive ou encore le raisin, la figue regorge de nombreux bienfaits et vertus. Riche en fibres, bêta-carotène, polyphénols et minéraux, celle-ci est à la fois tonifiante, nourrissante, antioxydante, reminéralisante, laxative et protectrice vasculaire. Très digeste, dégustez ce fruit miraculeux de multiples façons, qu’il soit cru, cuit ou sec."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.562871,"longitude":5.024779999999993}],"isMP3Activated":false,"radius":25,"type":7,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Head back down to Castagne square and take the road at the bottom on the left. A small alleyway makes it possible to reach the cemetery and Chapel of Saint Julien, the last leg of this adventure.","FR":"Redescendez sur la place Castagne et prenez la route en contrebas à gauche. Une petite ruelle vous permettra d’accéder au cimetière et à la chapelle Saint-Julien, dernière étape de l’aventure."},"arrayQuestions":{"EN":"Who am I? Proud of my sweet taste, enjoyed during the summer, growing in the garden, in your hands once I\'m rIpe, the perfect snack.","FR":"Qui suis-je ? Fière de mon goût sucré, Indémodable fruit de l’été, Grandissant dans mon jardin, Une fois bien mûre dans votre main, Est un régal tous les matins."},"arrayAnswers":{"EN":"Fig","FR":"Figue"},"arrayHints":{"EN":"I can be eaten raw and cooked","FR":"Observez bien ce poème, c\'est CAPITAL."},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/jardin"}},{"id":"PO132","arrayName":{"EN":"Step 13 - The cemetery and chapel of Saint-Julien","FR":"Etape 13 - Le cimetière et la chapelle Saint-Julien"},"arrayDescription":{"EN":"Did you know? Saint-Julien Chapel, located inside the current cemetery of Miramas-le-Vieux, is the oldest building in the city. Most likely built on the location of a pagan temple, transformed into a small Carolingian church, Saint-Julien Chapel dates back to the 12th century. This discrete Provencal Romanesque work of art is on the list of historic monuments since 1928. Today, it is used for religious funerals and musical performances.","FR":"Le saviez-vous ? La chapelle Saint-Julien, située à l’intérieur du cimetière actuel de Miramas-le-Vieux, est le bâtiment le plus ancien de la cité. Probablement édifiée à l’emplacement d’un temple païen puis transformée en petite église carolingienne, cette chapelle date du début du 12ème siècle. Cette discrète œuvre de l’art roman provençal figure dans l’inventaire des monuments historiques depuis 1928. Aujourd’hui, elle sert à l’occasion de funérailles religieuses ou de représentations musicales."},"arrayMP3":{},"arrayTextMP3":{},"arrayCoordinates":[{"latitude":43.561899,"longitude":5.025296999999995}],"isMP3Activated":false,"radius":45,"type":1,"isAutoDirectionAcivated":false,"status":4,"riddle":{"arrayInstructions":{"EN":"Mix the puzzle up for future adventurers.","FR":"Mélangez de nouveau le puzzle pour les futurs aventuriers."},"arrayQuestions":{"EN":"Go into the cemetery and head to the monument on the right side of Saint Julien Chapel. Take a good look... Then find the monument of the attached photo and look around, an error should stand out to you. Which one? (An uppercase letter)","FR":"Entrez dans le cimetière et dirigez-vous vers le monument qui se trouve à droite de la chapelle Saint-Julien. Regardez bien...  Trouvez ensuite le monument de la photo ci-jointe et observez-le bien, une erreur attirera votre attention. Laquelle ? (une lettre majuscule)"},"arrayAnswers":{"EN":"H","FR":"H"},"arrayHints":{"EN":"Concentrate on the names on the tombstones. ","FR":"Concentrez-vous sur les noms des pierres tombales"},"qrCodeUrl":"https://escapemind.wixsite.com/miramas/cimetiere"}}]}';

                  _navigationEngine.startNavigation(
                    route: route,
                    gpsColor: "#ff000000",
                    accessToken: "pk.eyJ1IjoibWlsZS1kZXYiLCJhIjoiY2s0OGJkNWEyMDB5ZTNna2Z5b2c2YTF3MCJ9.D5Ebdy0_ADSTQPTH4GCc9w",
                    mode: NavigationMode.TREASURE_HUNT,
                  );
                },
                child: Container(
                  height: 36,
                  width: 200,
                  color: Colors.blue,
                  child: Center(
                    child: Text(
                      "Start Game",
                      style: TextStyle(
                        color: Colors.white,
                      ),
                    ),
                  ),
                )
            ),

            Container(height: 40,),
            InkWell(
                onTap: () {
                  Navigator.push(context,
                    MaterialPageRoute(
                      builder: (context) => Navigation(mode: NavigationMode.NAVIGATE_IN_ROUTE),
                    ),
                  );
                },
                child: Container(
                  height: 36,
                  width: 200,
                  color: Colors.blue,
                  child: Center(
                    child: Text(
                      "Go To navigation widget",
                      style: TextStyle(
                        color: Colors.white,
                      ),
                    ),
                  ),
                )
            ),
            Container(height: 20,),
            InkWell(
                onTap: () {
                  Navigator.push(context,
                    MaterialPageRoute(
                      builder: (context) => Navigation(mode: NavigationMode.NAVIGATE_TO_POI),
                    ),
                  );
                },
                child: Container(
                  height: 36,
                  width: 200,
                  color: Colors.blue,
                  child: Center(
                    child: Text(
                      "Go navigation to poi widget",
                      style: TextStyle(
                        color: Colors.white,
                      ),
                    ),
                  ),
                )
            ),
          ],
        ),
    );
  }
}
